#ifndef BISON_VC_PARSE_H
# define BISON_VC_PARSE_H

# ifndef YYSTYPE
#  define YYSTYPE int
#  define YYSTYPE_IS_TRIVIAL 1
# endif
# define	TOK_BEGIN_VCARD	257
# define	TOK_END_VCARD	258
# define	TOK_GROUP	259
# define	TOK_NAME	260
# define	TOK_PARAM_NAME	261
# define	TOK_PARAM_VALUE	262
# define	TOK_VALUE	263


extern YYSTYPE yylval;

#endif /* not BISON_VC_PARSE_H */
